<?php

namespace Drupal\webform_remote_fields_api_tests_helper\Service;

use Drupal\webform_remote_fields\Service\WebformRemoteFieldsClientMiddleware;
use GuzzleHttp\Promise\Create;
use GuzzleHttp\Psr7\Response;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;

/**
 * Middleware to override the WebformRemoteFieldsClientMiddleware for testing.
 *
 * This middleware is used to simulate responses from a fake endpoint
 * instead of making actual HTTP requests.
 */
class WebformRemoteFieldsClientMiddlewareOverride extends WebformRemoteFieldsClientMiddleware {

  /**
   * Endpoint string to match for fake responses.
   *
   * @var string
   */
  const FAKE_ENDPOINTS = 'fake-webformremotefields.com';

  /**
   * Implements __invoke() function.
   */
  public function __invoke(callable $handler) {
    return function (
      RequestInterface $request,
      array $options,
    ) use ($handler) {
      // Modify the request object or options here if needed.
      // @codingStandardsIgnoreStart
      $uri = $request->getUri()->getHost();
      if (str_starts_with($uri, self::FAKE_ENDPOINTS)) {
        $responseData = self::loadFakeResponseBasedOnRequest($request);
        return Create::promiseFor(new Response(200, [], json_encode($responseData)));
      }

      return $handler($request, $options)->then(
        function (ResponseInterface $response) use ($request) {
          return $response;
        }
      );
    };
  }

  /**
   * Load fake response data based on the request.
   *
   * @param \Psr\Http\Message\RequestInterface $request
   *   The request object.
   *
   * @return array
   *   The fake response data.
   */
  private static function loadFakeResponseBasedOnRequest(RequestInterface $request): array {
    $path = $request->getUri()->getPath();
    if ($path === '/api/products') {
      return [
        [
          'id' => 1,
          'name' => 'Product 1',
        ],
        [
          'id' => 2,
          'name' => 'Product 2',
        ],
      ];
    }
    else if (str_starts_with($path, "/api/product/price")) {
      $id = str_replace("/api/product/price/", "", $path);
      $data = [
        1 => ['price' => 2.90],
        2 => ['price' => 7.50],
      ];
      return ['results' => $data[$id]];
    }
    else if (str_starts_with($path, "/api/product/discount")) {
      $quantity = str_replace("/api/product/discount/", "", $path);
      return [
        'discount' => (int) $quantity * 0.10,
      ];
    }
    else if (str_starts_with($path, "/api/product/colors")) {
      $product = str_replace("/api/product/colors/", "", $path);
      $data = [
        1 => [
          ['colorHash' => 'FFF', 'label' => 'Yellow'],
          ['colorHash' => '000', 'label' => 'Green'],
        ],
        2 => [
          ['colorHash' => 'FFF', 'label' => 'Yellow'],
          ['colorHash' => '000', 'label' => 'Green'],
          ['colorHash' => '010', 'label' => 'Blue'],
        ],
      ];
      return $data[$product];
    }
    else if (str_starts_with($path, "/api/product/sizes")) {
      $color = str_replace("/api/product/sizes/", "", $path);
      $data = [
        'FFF' => [
          ['label' => 'X', 'value' => 'x'],
          ['label' => 'G', 'value' => 'g'],
          ['label' => 'S', 'value' => 's'],
        ],
        '000' => [
          ['label' => 'G', 'value' => 'g'],
          ['label' => 'P', 'value' => 'p'],
        ],
      ];
      return $data[$color];
    }
    return [];
  }

}
