<?php

namespace Drupal\webform_remote_fields\Hook;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\webform\Entity\Webform;
use Drupal\webform\Plugin\WebformElementManagerInterface;
use Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper;

/**
 * Alter forms related to Webform Remote Fields.
 */
class FormAlterHook {

  public function __construct(
    private readonly RouteMatchInterface $routeMatch,
    private readonly WebformElementManagerInterface $webformElementManager,
    private readonly WebformRemoteFieldHelper $webformRemoteFieldHelper,
    private readonly AccountProxyInterface $currentUser,
  ) {}

  /**
   * Implements hook_form_webform_ui_element_delete_form_alter().
   *
   * Alters the Webform UI element delete form to restrict access based on user
   * permissions.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   The form state.
   * @param string $form_id
   *   The form ID.
   */
  #[Hook('form_webform_ui_element_delete_form_alter')]
  public function formWebformUiElementDelete(&$form, FormStateInterface $formState, $form_id): void {
    /** @var \Drupal\webform\Entity\Webform $webform */
    $webform = $this->routeMatch->getParameter('webform');
    $field_key = $this->routeMatch->getParameter('key');
    if (empty($webform) || empty($field_key)) {
      return;
    }

    $field_props = $webform->getElement($field_key);
    $elementType = $field_props['#webform_plugin_id'];

    $ourPlugins = array_filter($this->webformElementManager->getDefinitions(), function ($definition) {
      return isset($definition['provider']) && $definition['provider'] === 'webform_remote_fields';
    });

    // Do nothing for other plugins.
    if (!in_array($elementType, array_keys($ourPlugins))) {
      return;
    }

    if (!$this->webformRemoteFieldHelper->isRemoteFieldAdmin()) {
      unset($form["actions"]["submit"]);
      unset($form["warning"]);
      unset($form["description"]);
      $form['not_allowed'] = [
        "#type" => "markup",
        "#markup" => t('You do not have access to delete this field.'),
      ];
    }
  }

  /**
   * Implements hook_form_webform_ui_element_form_alter().
   *
   * Alters the Webform UI element form to restrict access based on user
   * permissions.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   The form state.
   * @param string $form_id
   *   The form ID.
   */
  #[Hook('form_webform_ui_element_form_alter')]
  public function formWebformUiElement(&$form, FormStateInterface $formState, $form_id) {
    $elementType = $form['properties']['type']['#value'];
    $ourPlugins = array_filter($this->webformElementManager->getDefinitions(), function ($definition) {
      return isset($definition['provider']) && $definition['provider'] === 'webform_remote_fields';
    });

    // Do nothing for other plugins.
    if (!in_array($elementType, array_keys($ourPlugins))) {
      return;
    }

    if (!$this->webformRemoteFieldHelper->isRemoteFieldAdmin()) {
      unset($form['properties']);
      unset($form['actions']);
      $form['not_allowed'] = [
        "#type" => "markup",
        "#markup" => "You do not have access to edit this field.",
      ];
    }
    // Denied template options.
    if (!$this->currentUser->hasPermission('administer webform_remote_fields template configuration')) {
      foreach (['process_engine', 'template_name', 'template_string'] as $key) {
        $form["properties"]["options"]["api"][$key]['#access'] = FALSE;
      }
    }
  }

  /**
   * Implements hook_form_webform_ui_element_type_select_form_alter().
   *
   * Alters the Webform UI element type select form to restrict access based on
   * user permissions.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   The form state.
   * @param string $form_id
   *   The form ID.
   */
  #[Hook('form_webform_handler_delete_form_alter')]
  public function formWebformHandlerDelete(&$form, FormStateInterface $formState, $form_id): void {
    if ($this->routeMatch->getParameter('webform_handler') !== 'webform_remote_fields_handler') {
      return;
    }
    if (!$this->webformRemoteFieldHelper->isRemoteFieldAdmin()) {
      unset($form['properties']);
      unset($form['actions']);
      unset($form["warning"]);
      unset($form["description"]);
      $form['not_allowed'] = [
        "#type" => "markup",
        "#markup" => "You do not have access to delete this handler.",
      ];
      return;
    }

    $webform = $this->routeMatch->getParameter('webform');
    if ($webform instanceof Webform && $this->webformRemoteFieldHelper->hasRemoteFields($webform)) {
      unset($form['properties']);
      unset($form['actions']);
      unset($form["warning"]);
      unset($form["description"]);
      $form['not_allowed'] = [
        "#type" => "markup",
        "#markup" => "You cannot delete this handler until all Webform Remote Field Elements are removed.",
      ];
    }
  }

  /**
   * Implements hook_form_webform_ui_element_type_select_form_alter().
   *
   * Alters the Webform UI element type select form to restrict access
   * to certain sections based on user permissions.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   The form state.
   */
  #[Hook('form_webform_ui_element_type_select_form_alter')]
  public function formWebformUiElementTypeSelect(&$form, FormStateInterface $formState) {
    if ($this->webformRemoteFieldHelper->isRemoteFieldAdmin()) {
      return;
    }
    $unavailable_sections = [
      'Remote Fields',
    ];
    foreach ($unavailable_sections as $section) {
      $id = $this->findFormContainerElementByTitle($form, $section);
      unset($form[$id]);
    }
  }

  /**
   * Find the index of a form item within a form array by its title.
   *
   * @param array $form
   *   The form array to search within.
   * @param string $title
   *   The title of the form item to find.
   *
   * @return int|bool
   *   The index of the form item if found, or FALSE if not found.
   */
  protected function findFormContainerElementByTitle(array &$form, string $title): ?int {
    foreach ($form as $key => $element) {
      if (empty($element['#title'])) {
        continue;
      }
      // Check if the current element has a '#title' property and if it matches
      // the desired title.
      if ($element['#title'] == $title) {
        return $key;
      }
      // If the current element has children (sub-elements), recursively search
      // within them.
      if ($element['#type'] == 'container') {
        $result = $this->findFormContainerElementByTitle($element, $title);
        if ($result !== FALSE) {
          return $result;
        }
      }
    }
    // If the title is not found in the current form array, return FALSE.
    return FALSE;
  }

}
