<?php

namespace Drupal\webform_remote_fields\Hook;

use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Render\BubbleableMetadata;

/**
 * Implement token hooks for Webform Remote Fields.
 */
class TokenHook {

  /**
   * Implements hook_tokens().
   *
   * This hook provides dynamic tokens for Webform Remote Fields.
   *
   * @param string $type
   *   The type of tokens being processed (e.g., 'form_value').
   * @param array $tokens
   *   An associative array of tokens to be replaced, where the key is the token
   *   name and the value is the original token string.
   * @param array $data
   *   Additional data that may be used to replace tokens, such as form values.
   * @param array $options
   *   Options for token processing, such as whether to ignore missing tokens.
   * @param \Drupal\Core\Render\BubbleableMetadata $bubbleable_metadata
   *   Metadata for bubbleable render arrays, allowing the tokens to be.
   */
  #[Hook('tokens')]
  public function tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata): array {
    $replacements = [];
    if ($type == 'form_value' && !empty($data['form_value'])) {
      $formValues = $data['form_value'];
      foreach ($tokens as $name => $original) {
        $values = explode(':', $name);
        if (count($values) == 2) {
          $dynamicName = $values[1];
          $replacements[$original] = $formValues[$dynamicName];
        }
      }
    }
    return $replacements;
  }

  /**
   * Implements hook_token_info().
   *
   * This hook provides information about the tokens available for
   * Webform Remote Fields.
   *
   * @return array
   *   An associative array containing token types and their definitions.
   */
  #[Hook('token_info')]
  public function tokenInfo(): array {
    $tokens = [];
    $type = [
      'name' => 'Webform Remote Fields',
      'description' => t('Tokens related with Webform elements with the remote fields.'),
    ];
    $tokens['name'] = [
      'dynamic' => TRUE,
      'name' => 'Form value',
      'description' => t('Use any Webform elements values in the remote fields settings.'),
    ];
    return [
      'types' => ['form_value' => $type],
      'tokens' => ['form_value' => $tokens],
    ];
  }

}
