<?php

namespace Drupal\webform_remote_fields\Plugin\WebformHandler;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\Utility\WebformElementHelper;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform_remote_fields\Ajax\RemoteFieldTriggerChangeValue;
use Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Custom API Webform Handler.
 *
 * @WebformHandler(
 *   id = "webform_remote_fields_handler",
 *   label = @Translation("Webform Remote Fields Handler"),
 *   category = @Translation("Remote Fields"),
 *   description = @Translation("Handles the custom API fields and special
 *   alters."), cardinality =
 *   \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_IGNORED,
 * )
 */
class WebformRemoteFieldsHandler extends WebformHandlerBase {

  /**
   * The webform remote field helper service.
   *
   * @var \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper
   */
  protected WebformRemoteFieldHelper $webformRemoteFieldHelper;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected CacheBackendInterface $cacheBackend;

  /**
   * The time service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected TimeInterface $time;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->webformRemoteFieldHelper = $container->get(WebformRemoteFieldHelper::class);
    $instance->cacheBackend = $container->get('cache.webform_remote_fields');
    $instance->time = $container->get('datetime.time');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function alterForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission): void {
    if (empty($form['elements'])) {
      return;
    }
    $fieldsThatDependsOnOtherFields = $this->getElementsThatUseOtherFieldsOnApi($form);
    if (empty($fieldsThatDependsOnOtherFields)) {
      return;
    }

    foreach ($fieldsThatDependsOnOtherFields as $element_id => $blockedElements) {
      // Before add the ajax command, we need to check if there is any
      // dependent element on the current page.
      // Otherwise, we don't need to add the ajax command to avoid making
      // unnecessary ajax calls.
      $currentStep = $form["progress"]["#current_page"] ?? NULL;
      if ($currentStep && !$this->hasAnyBlockedElementPresent($currentStep, $blockedElements, $form)) {
        continue;
      }
      $element = &WebformElementHelper::getElement($form['elements'], $element_id);
      // Add AJAX to every element that has been set as "dependency" or API
      // param in another element.
      $element['#ajax'] = [
        'callback' => [$this, 'ajaxCallback'],
        'event' => 'change',
        'progress' => ['type' => 'throbber'],
      ];
      $fieldName = $element['#name'] ?? $element["#webform_key"];
      $element['#attributes']['data-blocks-this-element'] = $blockedElements;
      $triggeredElement = $form_state->getTriggeringElement();
      if (!empty($triggeredElement)) {
        $triggeredElementId = $triggeredElement['#name'] ?? $triggeredElement["#webform_key"];
        if ($triggeredElementId === $fieldName) {
          $form_state->setTriggeringElement($element);
        }
      }
    }
  }

  /**
   * AJAX callback function.
   */
  public function ajaxCallback(array &$form, FormStateInterface $formState) {
    $blockedElementMachineName = $formState->getTriggeringElement()['#attributes']['data-blocks-this-element'];
    $dependentFieldsOfTheFieldUpdated = $this->getElementsThatUseOtherFieldsOnApi($form);
    $response = new AjaxResponse();
    foreach ($blockedElementMachineName as $key) {
      $element = &WebformElementHelper::getElement($form['elements'], $key);
      // Update an element only if it's rendered on the page.
      if (!$element['#access']) {
        continue;
      }

      $this->addAjaxCommand($response, $element);

      if (!isset($dependentFieldsOfTheFieldUpdated[$key])) {
        continue;
      }

      $dependentFields = $dependentFieldsOfTheFieldUpdated[$key];
      if (!empty($dependentFields)) {
        foreach ($dependentFields as $dependentKey) {
          $dependentElement = &WebformElementHelper::getElement($form['elements'], $dependentKey);
          if (!$dependentElement['#access']) {
            continue;
          }
          $this->addAjaxCommand($response, $dependentElement);
        }
      }
    }
    return $response;
  }

  /**
   * Add the command ajax of an element into the ajax response.
   *
   * @param \Drupal\Core\Ajax\AjaxResponse $response
   *   The response to have the command added.
   * @param array $element
   *   The element to be updated.
   */
  private function addAjaxCommand(AjaxResponse &$response, array $element): void {
    $id = $dependentElement['#name'] ?? $element['#webform_key'];
    $response->addCommand(new ReplaceCommand('#' . $id . '-wrapper', $element));
    if (!empty($element["#name"])) {
      $response->addCommand(new RemoteFieldTriggerChangeValue('[name="' . $element["#name"] . '"]'));
    }
  }

  /**
   * Get elements id that use other fields from cache.
   */
  protected function getElementsThatUseOtherFieldsOnApi($form): array {
    $webformId = $form['#webform_id'];
    $cid = 'wr:dependent_fields:' . $webformId;
    if ($data = $this->cacheBackend->get($cid)) {
      return $data->data;
    }
    $elements = $this->webformRemoteFieldHelper->getElementsThatUseOtherFieldsOnApi($form['elements']);
    $this->cacheBackend->set($cid, $elements, $this->time->getCurrentTime() + 86400, ['config:webform.webform.' . $webformId]);
    return $elements;
  }

  /**
   * Check if any blocked element is present in the current step.
   *
   * @param string $currentStep
   *   The current step of the form.
   * @param array $blockedElements
   *   The blocked elements to check.
   * @param array $form
   *   The form array.
   *
   * @return bool
   *   TRUE if any blocked element is present, FALSE otherwise.
   */
  protected function hasAnyBlockedElementPresent(string $currentStep, array $blockedElements, array $form): bool {
    foreach ($blockedElements as $blockedElementId) {
      $blockedElement = WebformElementHelper::getElement($form['elements'], $blockedElementId);
      if ($currentStep !== NULL && in_array($currentStep, $blockedElement['#webform_parents'])) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
