<?php

namespace Drupal\webform_remote_fields\Service;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Utility\Token;
use Drupal\webform\Entity\Webform;
use Symfony\Component\Yaml\Yaml;

/**
 * Helper class to provide feature about remote field elements.
 */
class WebformRemoteFieldHelper {

  public function __construct(
    private readonly PluginManagerInterface $pluginManager,
    private readonly Token $token,
    private readonly AccountInterface $currentUser,
  ) {}

  /**
   * Get all custom remote field elements.
   *
   * @return array
   *   An array with elements.
   */
  public function getElements(): array {
    return array_filter($this->pluginManager->getDefinitions(), function ($definition) {
      return isset($definition['provider']) && $definition['provider'] === 'webform_remote_fields';
    });
  }

  /**
   * Get fields from token.
   *
   * @param string $source
   *   The string to scan the tokens.
   *
   * @return array
   *   Array of field.
   */
  public function getFieldsFromToken(string $source): array {
    $fields = [];
    $tokens = $this->token->scan($source);
    if (empty($tokens['form_value'])) {
      return $fields;
    }

    foreach ($tokens['form_value'] as $fieldName => $value) {
      $values = array_filter(explode(':', $fieldName));
      if (count($values) === 2) {
        $fields[] = $values[1];
      }
    }
    return $fields;
  }

  /**
   * Build the header replacing token for the values.
   *
   * @param string $headersSettings
   *   The header settings.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   Form state data.
   *
   * @return array|null
   *   Array of header options or null if a field is not completed.
   */
  public function buildHeaderRequestOptions(string $headersSettings, FormStateInterface $formState): array|null {
    $headers = [];
    $headerLine = array_filter(explode("\n", $headersSettings));
    if (empty($headerLine)) {
      return [];
    }
    foreach ($headerLine as $headerValue) {
      $fields = $this->getFieldsFromToken($headerValue);
      // If any token found adds as plain text in header props.
      if (empty($fields)) {
        [$headerKey, $headerKeyValue] = explode(':', $headerValue);
        $headers[trim($headerKey)] = trim($headerKeyValue);
        continue;
      }
      foreach ($fields as $fieldName) {
        $fieldValue = $formState->getValue($fieldName);
        if (empty($fieldValue)) {
          return NULL;
        }
        $headerLineReplaced = $this->token->replace($headerValue, ['form_value' => $formState->getValues()]);
        [$headerKey, $headerKeyValue] = explode(':', $headerLineReplaced);
        $headers[trim($headerKey)] = trim($headerKeyValue);
      }
    }
    return $headers;
  }

  /**
   * Build query params replacing tokens to the values.
   *
   * @param array $params
   *   Params defined to the request.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   Form state.
   *
   * @return array|null
   *   Array of query options or null if a field is not completed.
   */
  public function buildQueryParamsRequestTokens(array $params, FormStateInterface $formState): array|NULL {
    $queryParams = [];
    foreach ($params as $param) {

      // Replace if need param_name.
      $paramName = $param["param_name"];
      $tokens = $this->token->scan($param["param_name"]);
      if (!empty($tokens['form_value'])) {
        foreach ($tokens['form_value'] as $fieldName => $token) {
          $fieldName = explode(":", $fieldName)[1];
          $fieldValue = $formState->getValue($fieldName);
          if (empty($fieldValue)) {
            return NULL;
          }
          $paramName = $this->token->replace($paramName, ['form_value' => $formState->getValues()]);
        }
      }

      // Replace if need param_value.
      $paramValue = $param["param_value"];
      $tokens = $this->token->scan($param["param_value"]);
      if (!empty($tokens['form_value'])) {
        foreach ($tokens['form_value'] as $fieldName => $token) {
          $fieldName = explode(":", $fieldName)[1];
          $fieldValue = $formState->getValue($fieldName);
          if (empty($fieldValue)) {
            return NULL;
          }
          $paramValue = $this->token->replace($paramValue, ['form_value' => $formState->getValues()]);
        }
      }
      $queryParams[$paramName] = $paramValue;
    }
    return $queryParams;
  }

  /**
   * Build the endpoint replacing token for the values.
   *
   * @param string $endpoint
   *   The endpoint settings.
   * @param array $values
   *   Values to replace in the endpoint.
   *
   * @return string|null
   *   The endpoint with replaced values or null if a field is not completed.
   */
  public function buildEndpointUrl(string $endpoint, array $values = []): ?string {
    // Build an endpoint if it has tokens.
    $endpointsFields = $this->getFieldsFromToken($endpoint);
    if (!empty($endpointsFields)) {
      foreach ($endpointsFields as $fieldName) {
        $value = $values[$fieldName];
        if (empty($value)) {
          return NULL;
        }
      }
      $endpoint = $this->token->replace($endpoint, ['form_value' => $values]);
    }
    return $endpoint;
  }

  /**
   * Build the request options.
   *
   * @param bool $needs_header
   *   If the request needs headers.
   * @param array $element
   *   The webform element.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   An object of the form state.
   *
   * @return array|null
   *   Array with the request options or null if a field is not completed.
   */
  public function buildRequestOptions(bool $needs_header, array $element, FormStateInterface $formState): array|NULL {
    $requestOptions = [];
    // Build headers.
    if ($needs_header) {
      $headers = $this->buildHeaderRequestOptions($element['#headers'], $formState);
      if (is_null($headers)) {
        return NULL;
      }
      if (!empty($headers)) {
        $requestOptions['headers'] = $headers;
      }
    }

    // Check query params.
    if (!empty($element['#api_endpoint_params'])) {
      $queryParams = $this->buildQueryParamsRequestTokens(
        $element['#api_endpoint_params'],
        $formState
      );
      if (is_null($queryParams)) {
        return NULL;
      }
      if (!empty($queryParams)) {
        $requestOptions['query'] = $queryParams;
      }
    }
    return $requestOptions;
  }

  /**
   * Get elements id that use other fields.
   *
   * @param array $elements
   *   List of webform elements.
   *
   * @return array
   *   Return an array with the keys of elements that use other fields,
   *   followed by the field name that depends on it.
   */
  public function getElementsThatUseOtherFieldsOnApi(array $elements = []): array {
    if (empty($elements)) {
      return [];
    }
    $elementIds = [];
    $elementTypes = array_keys($this->getElements());

    foreach ($elements as $key => $element) {

      if (empty($element['#type'])) {
        continue;
      }

      // If the current element has children (sub-elements), recursively search
      // within them and merge results.
      if (is_array($element) && $this->isContainerElement($element['#type'])) {
        $elementIds = array_merge_recursive($elementIds, $this->getElementsThatUseOtherFieldsOnApi($element));
      }

      // Check dependencies only for the remote fields.
      if (!in_array($element['#type'], $elementTypes)) {
        continue;
      }

      if (!empty($element['#api_endpoint_params'])) {
        // Get elements defined as parameter.
        foreach ($element['#api_endpoint_params'] as $param) {
          $hasToken = $this->token->scan($param['param_value'] . ' ' . $param['param_name']);
          if (empty($hasToken)) {
            continue;
          }

          $tokens = $this->token->scan($param['param_value']);
          if (!empty($tokens['form_value'])) {
            foreach ($tokens['form_value'] as $fieldName => $value) {
              $values = explode(':', $fieldName);
              if (count($values) === 2) {
                $dependent_field = $values[1];
                $elementIds[$dependent_field][] = $key;
              }
            }
          }

          $tokens = $this->token->scan($param['param_name']);
          if (!empty($tokens['form_value'])) {
            foreach ($tokens['form_value'] as $fieldName => $value) {
              $values = explode(':', $fieldName);
              if (count($values) === 2) {
                $dependent_field = $values[1];
                $elementIds[$dependent_field][] = $key;
              }
            }
          }
        }
      }

      $settings = [
        '#api_endpoint',
        '#api_result_select_value',
        '#api_result_select_text',
        '#headers',
      ];

      foreach ($settings as $setting) {
        if (empty($element[$setting])) {
          continue;
        }
        $tokens = $this->token->scan($element[$setting]);
        if (!empty($tokens['form_value'])) {
          foreach ($tokens['form_value'] as $dependent_field => $token) {
            $values = explode(':', $dependent_field);
            if (count($values) === 2) {
              $dependent_field = $values[1];
              $elementIds[$dependent_field][] = $key;
            }
          }
        }
      }
    }

    return $elementIds;
  }

  /**
   * Get the container types.
   *
   * @return array
   *   The container types.
   */
  public function getContainerTypes(): array {
    return ['container', 'webform_wizard_page', 'fieldset', 'details', 'fieldset', 'webform_flexbox', 'webform_section'];
  }

  /**
   * Check if the element is a container.
   *
   * @param string $elementType
   *   The element type.
   *
   * @return bool
   *   TRUE if the element is a container, FALSE otherwise.
   */
  public function isContainerElement(string $elementType): bool {
    return in_array($elementType, $this->getContainerTypes());
  }

  /**
   * Extract setting of the Webform remote field element.
   *
   * The order of array with the setting values.
   *  -> api_endpoint_add_query_parameter.
   *  -> api_endpoint.
   *  -> api_result_select_text.
   *  -> headers.
   *
   * @param array $element
   *   The webform element.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   An object of the form state.
   *
   * @return array
   *   Array with the key => value setting.
   */
  public function extractElementSettings(array $element, FormStateInterface $formState): array {
    $apiEndpointAddQueryParameter = $element['#api_endpoint_add_query_parameter'] ?? FALSE;
    $apiEndpoint = $element['#api_endpoint'] ?? NULL;
    $apiResultSelectValue = $element['#api_result_select_value'] ?? NULL;
    $apiResultSelectText = $element['#api_result_select_text'] ?? NULL;
    $apiNeedsHeader = $element['#needs_header'] ?? NULL;
    $rootPropertyPath = $element['#root_property_path'] ?? NULL;

    $custom_headers = [];
    if ($apiNeedsHeader) {
      $custom_headers = Yaml::parse($element['#headers']);
    }

    // As we have the support for get the property request value from,
    // another webform field. We have to replace the tokens for
    // #api_result_select_value setting at this point.
    if (!empty($element['#api_result_select_value'])) {
      $apiResultSelectValueToken = $this->token->replace(
        $element['#api_result_select_value'],
        ['form_value' => $formState->getValues()],
      );
      if ($apiResultSelectValueToken) {
        $apiResultSelectValue = $apiResultSelectValueToken;
      }
    }

    return [
      'api_endpoint_add_query_parameter' => $apiEndpointAddQueryParameter,
      'api_endpoint' => $apiEndpoint,
      'root_property_path' => $rootPropertyPath,
      'api_result_select_value' => $apiResultSelectValue,
      'api_result_select_text' => $apiResultSelectText,
      'custom_headers' => $custom_headers,
      'needs_header' => $apiNeedsHeader,
      'api_round_result' => $element['#api_round_result'] ?? 0,
    ];
  }

  /**
   * Check if the element is present on the current page.
   *
   * If there is no progress element, it means that the form is not paginated,
   * so the element is always present, and we return TRUE.
   *
   * @param array $element
   *   The webform element.
   * @param array $form
   *   The webform form.
   *
   * @return bool
   *   TRUE if the element is present in the current page, FALSE otherwise.
   */
  public function elementIsPresentInCurrentPage($element, $form) {
    if (!isset($form["progress"]["#current_page"])) {
      return TRUE;
    }
    $currentStep = $form["progress"]["#current_page"];
    if ($currentStep && !empty($element['#webform_parents'])) {
      return in_array($currentStep, $element['#webform_parents']);
    }
    return FALSE;
  }

  /**
   * Check if the webform has remote fields.
   *
   * @param \Drupal\webform\Entity\Webform $webform
   *   The webform entity.
   *
   * @return bool
   *   TRUE if the webform has remote fields, FALSE otherwise.
   */
  public function hasRemoteFields(Webform $webform): bool {
    $remoteElements = $this->getElements();
    $elements = $webform->getElementsDecodedAndFlattened();
    if (empty($elements)) {
      return FALSE;
    }
    $elementsType = array_column($elements, '#type');
    return !empty(array_intersect(array_unique($elementsType), array_keys($remoteElements)));
  }

  /**
   * Check if the current user is an admin for remote fields.
   *
   * @return bool
   *   TRUE if the user has permission to administer remote fields,
   *   FALSE otherwise.
   */
  public function isRemoteFieldAdmin(): bool {
    return $this->currentUser->hasPermission('administer webform_remote_fields configuration');
  }

}
