<?php

namespace Drupal\webform_remote_fields\Traits;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform_remote_fields\Event\WebformRemoteFieldAlterResponseEvent;
use Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper;

/**
 * Provide shared features between Webform Remote Field elements.
 */
trait RemoteFieldBaseTrait {

  /**
   * {@inheritDoc}
   */
  public function getInfo() {
    // @phpstan-ignore-next-line
    $info = parent::getInfo();
    $info['#process'][] = [
      static::class,
      'processLibrary',
    ];
    return $info;
  }

  /**
   * Get webform remote field helper service.
   */
  public static function getFieldHelper(): WebformRemoteFieldHelper {
    return \Drupal::service(WebformRemoteFieldHelper::class);
  }

  /**
   * Attach a library to trigger element change.
   */
  public static function processLibrary($element, FormStateInterface $form_state, $form): array {
    $element['#attached']['library'][] = 'webform_remote_fields/webform_remote_fields';
    return $element;
  }

  /**
   * Make the requests for each remote field element.
   */
  protected static function processElementRequest(&$element, FormStateInterface $formState): array {
    $wrHelperService = static::getFieldHelper();

    $result = [
      'error' => FALSE,
      'data' => NULL,
    ];

    // Get api settings from the element to make the request.
    $apiSettings = $wrHelperService->extractElementSettings($element, $formState);

    // If value or text|label to get from the request, it's a token,
    // replace it for the value.
    // Otherwise, do not make the request until the value is filled.
    $resultSelectTextField = $wrHelperService->getFieldsFromToken($apiSettings["api_result_select_text"] ?? "");
    if (!empty($resultSelectTextField)) {
      $value = $formState->getValue($resultSelectTextField[0]);
      if (empty($value)) {
        return $result;
      }
      $element['#api_result_select_text'] = $value;
    }
    $resultSelectValueField = $wrHelperService->getFieldsFromToken($apiSettings["api_result_select_value"] ?? "");
    if (!empty($resultSelectValueField)) {
      $value = $formState->getValue($resultSelectValueField[0]);
      if (empty($value)) {
        return $result;
      }
      $element['#api_result_select_value'] = $value;
    }

    // Get request options and the endpoint, if null,
    // the request will not be made because the required values are not filled.
    $requestOptions = $wrHelperService->buildRequestOptions(!empty($apiSettings["needs_header"]), $element, $formState);
    if (is_null($requestOptions)) {
      return $result;
    }
    $endpoint = $wrHelperService->buildEndpointUrl($apiSettings['api_endpoint'], $formState->getValues());
    if (is_null($endpoint)) {
      return $result;
    }

    // Make the request.
    /** @var \Drupal\webform_remote_fields\Service\HandlerRequest $handler */
    $handler = \Drupal::service('webform_remote_fields.handler_request');
    $result['data'] = $handler->get($endpoint, $requestOptions, [
      'form_state' => $formState,
      'element' => $element,
    ]);

    // Check if a specific root property path needs to be extracted.
    if (!empty($apiSettings['root_property_path']) && is_array($result['data'])) {
      $result['data'] = static::getDataFromResponse($apiSettings['root_property_path'], $result['data']);
    }

    // Allow other modules to alter the response before set the value.
    $event = new WebformRemoteFieldAlterResponseEvent(
      $formState,
      $element,
      $result['data'] ?? [],
    );
    \Drupal::service('event_dispatcher')
      ->dispatch($event, WebformRemoteFieldAlterResponseEvent::EVENT_NAME);

    $result['data'] = $event->getResponse();
    if (is_null($result['data'])) {
      $result['error'] = TRUE;
    }
    return $result;
  }

  /**
   * Transform #api_endpoint_params setting to array.
   *
   * @param array $element
   *   Webform element.
   *
   * @return array
   *   Array where parameter value is the key, param value it's the array value.
   */
  public static function apiEndpointParametersToArray(array $element): array {
    if (empty($element['#api_endpoint_params'])) {
      return [];
    }
    $map = [];
    foreach ($element['#api_endpoint_params'] as $param) {
      $map[$param['param_name']] = $param['param_value'];
    }
    return $map;
  }

  /**
   * Check if the element is triggered.
   */
  public static function isElementTriggered(array $element, FormStateInterface $formState): bool {
    $triggeredElement = $formState->getTriggeringElement();
    if (empty($triggeredElement)) {
      return FALSE;
    }
    return $element['#webform_key'] === $triggeredElement['#webform_key'];
  }

  /**
   * Get data from the response based on the root property path.
   *
   * @param string|null $root_property_path
   *   The root property path to extract data from the response.
   * @param mixed $response
   *   The response data.
   *
   * @return mixed
   *   The extracted data or an empty array if not found.
   */
  public static function getDataFromResponse(?string $root_property_path, array $response = []): mixed {
    if (empty($root_property_path)) {
      return $response;
    }
    $root_path_split = explode('.', $root_property_path);
    $data = $response;
    foreach ($root_path_split as $path) {
      if (isset($data[$path])) {
        $data = $data[$path];
      }
      else {
        return [];
      }
    }
    return $data;
  }

}
