<?php

namespace Drupal\Tests\webform_remote_fields\Kernel;

use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper;

/**
 * Test service WebformRemoteFieldHelper.
 *
 * @covers \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper
 * @group webform_remote_fields
 */
class WebformRemoteFieldHelperTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'webform_remote_fields',
    'webform',
    'user',
  ];

  /**
   * Service to test.
   *
   * @var \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper|mixed
   */
  protected WebformRemoteFieldHelper $webformRemoteFieldHelper;

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();
    $this->webformRemoteFieldHelper = \Drupal::service(WebformRemoteFieldHelper::class);
  }

  /**
   * Get fields from token.
   *
   * @dataProvider dataProviderGetFieldsFromToken
   * @covers \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper::getFieldsFromToken
   */
  public function testGetFieldsFromToken(string $source, array $expectedResult) {
    $result = $this->webformRemoteFieldHelper->getFieldsFromToken($source);
    $this->assertEquals($expectedResult, $result);
  }

  /**
   * Test buildRequestOptions.
   *
   * @dataProvider dataProviderBuildRequestOptions
   * @covers \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper::buildRequestOptions
   */
  public function testBuildRequestOptions($needsHeader, $element, $formState, $expectedResult) {
    $result = $this->webformRemoteFieldHelper
      ->buildRequestOptions($needsHeader, $element, $formState);
    $this->assertEquals($expectedResult, $result);
  }

  /**
   * Test buildHeaderRequestOptions.
   *
   * @dataProvider dataProviderBuildHeaderRequestOptions
   * @covers \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper::buildHeaderRequestOptions
   */
  public function testBuildHeaderRequestOptions($element, FormStateInterface $formState, $expectedResult) {
    $result = $this->webformRemoteFieldHelper
      ->buildHeaderRequestOptions($element, $formState);
    $this->assertEquals($expectedResult, $result);
  }

  /**
   * Get build query params.
   *
   * @dataProvider dataProviderBuildQueryParamsRequestTokens
   * @covers \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper::buildQueryParamsRequestTokens
   */
  public function testBuildQueryParamsRequestTokens(array $params, array $formStateValues, $expectedResult) {
    $formStateMock = new FormState();
    foreach ($formStateValues as $field => $value) {
      $formStateMock->setValue($field, $value);
    }

    $result = $this->webformRemoteFieldHelper
      ->buildQueryParamsRequestTokens($params, $formStateMock);
    $this->assertEquals($expectedResult, $result);
  }

  /**
   * Test buildEndpointUrl.
   *
   * @dataProvider dataProviderBuildEndpointUrl
   * @covers \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper::buildEndpointUrl
   */
  public function testBuildEndpointUrl(string $endpoint, array $formStateValues, string|null $expectedResult) {
    $result = $this->webformRemoteFieldHelper
      ->buildEndpointUrl($endpoint, $formStateValues);
    $this->assertEquals($expectedResult, $result);
  }

  /**
   * Test getElementsThatUseOtherFieldsOnApi.
   *
   * @dataProvider dataProviderGetElementsThatUseOtherFieldsOnApi
   * @covers \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper::getElementsThatUseOtherFieldsOnApi
   */
  public function testGetElementsThatUseOtherFieldsOnApi(array $elements, array $expectedResult) {
    $result = $this->webformRemoteFieldHelper
      ->getElementsThatUseOtherFieldsOnApi($elements);
    $this->assertEquals($expectedResult, $result);
  }

  /**
   * Test extractElementSettings.
   *
   * @dataProvider dataProviderExtractElementSettings
   * @covers \Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper::extractElementSettings
   */
  public function testExtractElementSettings(array $element, array $formValues, array $expectedResult) {
    $formState = new FormState();
    foreach ($formValues as $key => $value) {
      $formState->setValue($key, $value);
    }

    $result = $this->webformRemoteFieldHelper->extractElementSettings($element, $formState);
    $this->assertEquals($expectedResult, $result);
  }

  /**
   * Data provider for getFieldsFromToken.
   *
   * @return \Generator
   *   Value to test.
   */
  public static function dataProviderGetFieldsFromToken(): \Generator {
    yield [
      '',
      [],
    ];
    yield [
      '[form_value:name:]',
      [],
    ];
    yield [
      '[form_value:name:some_field]',
      ['some_field'],
    ];
    yield [
      'http://example.com/endpoint-x/[form_value:name:some_field]/another-path/[form_value:name:second_field]',
      ['some_field', 'second_field'],
    ];
  }

  /**
   * Data provider for testBuildHeaderRequestOptions.
   *
   * @return \Generator
   *   Data to test.
   */
  public static function dataProviderBuildHeaderRequestOptions(): \Generator {
    $formState = new FormState();
    $formState->setValue('basic_auth', 'basic_auth_value_example');
    yield [
      "Authorization: [form_value:name:basic_auth]\nAnother-Header: some value",
      $formState,
      [
        'Authorization' => 'basic_auth_value_example',
        'Another-Header' => 'some value',
      ],
    ];
    yield [
      "Authorization: [form_value:name:basic_auth]\nAnother-Header: some value",
      new FormState(),
      NULL,
    ];
    yield [
      "Authorization: bearer token\nAnother-Header: some value",
      new FormState(),
      [
        'Authorization' => 'bearer token',
        'Another-Header' => 'some value',
      ],
    ];
  }

  /**
   * Data provider for testBuildQueryParamsRequestTokens.
   *
   * @return array[]
   *   Data to test.
   */
  public static function dataProviderBuildQueryParamsRequestTokens(): array {
    return [
      'no tokens in params' => [
        'params' => [
          ['param_name' => 'static_name', 'param_value' => 'static_value'],
        ],
        'formStateValues' => [],
        'expectedResult' => ['static_name' => 'static_value'],
      ],
      'token in param_name' => [
        'params' => [
          [
            'param_name' => '[form_value:name:field1]',
            'param_value' => 'static_value',
          ],
        ],
        'formStateValues' => ['field1' => 'replaced_name'],
        'expectedResult' => ['replaced_name' => 'static_value'],
      ],
      'token in param_value' => [
        'params' => [
          [
            'param_name' => 'static_name',
            'param_value' => '[form_value:name:field1]',
          ],
        ],
        'formStateValues' => ['field1' => 'replaced_value'],
        'expectedResult' => ['static_name' => 'replaced_value'],
      ],
      'token not replaced due to empty field' => [
        'params' => [
          [
            'param_name' => '[form_value:name:field1]',
            'param_value' => 'static_value',
          ],
        ],
        'formStateValues' => [],
        'expectedResult' => NULL,
      ],
      'multiple tokens in params' => [
        'params' => [
          [
            'param_name' => '[form_value:name:field1]',
            'param_value' => '[form_value:name:field2]',
          ],
        ],
        'formStateValues' => [
          'field1' => 'replaced_name',
          'field2' => 'replaced_value',
        ],
        'expectedResult' => ['replaced_name' => 'replaced_value'],
      ],
    ];
  }

  /**
   * Data provider for testBuildRequestOptions.
   *
   * @return \Generator
   *   Data to test.
   */
  public static function dataProviderBuildRequestOptions(): \Generator {
    $formStateFilled = new FormState();
    $formStateFilled->setValue('field_1', 'value_1');
    $formStateFilled->setValue('field_2', 'value_2');

    $fieldOneNotFilled = new FormState();
    $fieldOneNotFilled->setValue('field_2', 'value_2');

    yield 'empty, no tokens defined' => [
      FALSE,
      [
        '#headers' => 'no matter since needsHeader is false',
        '#api_endpoint_params' => '',
      ],
      new FormState(),
      [],
    ];
    yield 'headers with static values' => [
      TRUE,
      [
        '#headers' => "Authorization: bearer token here\nAnother-Header: some value",
        '#api_endpoint_params' => '',
      ],
      new FormState(),
      [
        'headers' => [
          'Authorization' => 'bearer token here',
          'Another-Header' => 'some value',
        ],
      ],
    ];
    yield 'headers with token values replaced' => [
      TRUE,
      [
        '#headers' => "Authorization: [form_value:name:field_1]",
        '#api_endpoint_params' => '',
      ],
      $formStateFilled,
      [
        'headers' => [
          'Authorization' => 'value_1',
        ],
      ],
    ];
    yield 'query params replaced with tokens and static values, header empty.' => [
      TRUE,
      [
        '#headers' => "",
        '#api_endpoint_params' => [
          [
            'param_name' => 'query_param_1_static',
            'param_value' => '[form_value:name:field_1]',
          ],
          [
            'param_name' => '[form_value:name:field_1]',
            'param_value' => 'param name dynamic with static value',
          ],
          [
            'param_name' => '[form_value:name:field_2]',
            'param_value' => '[form_value:name:field_1]',
          ],
        ],
      ],
      $formStateFilled,
      [
        'query' => [
          'query_param_1_static' => 'value_1',
          'value_1' => 'param name dynamic with static value',
          'value_2' => 'value_1',
        ],
      ],
    ];
    yield 'headers and query with token replaced' => [
      TRUE,
      [
        '#headers' => "Authorization: [form_value:name:field_1]",
        '#api_endpoint_params' => [
          [
            'param_name' => 'query_param_1_static',
            'param_value' => '[form_value:name:field_2]',
          ],
        ],
      ],
      $fieldOneNotFilled,
      NULL,
    ];
    yield 'null since endpoint params has required field not filled' => [
      TRUE,
      [
        '#headers' => "Authorization: [form_value:name:field_2]",
        '#api_endpoint_params' => [
          [
            'param_name' => 'query_param_1_static',
            'param_value' => '[form_value:name:field_1]',
          ],
        ],
      ],
      $fieldOneNotFilled,
      NULL,
    ];
    yield 'null since header has required field not filled' => [
      TRUE,
      [
        '#headers' => "Authorization: [form_value:name:field_1]",
        '#api_endpoint_params' => [
          [
            'param_name' => 'query_param_1_static',
            'param_value' => '[form_value:name:field_1]',
          ],
        ],
      ],
      $fieldOneNotFilled,
      NULL,
    ];
  }

  /**
   * Data provider for testBuildEndpointUrl.
   *
   * @return \Generator
   *   Data to test.
   */
  public static function dataProviderBuildEndpointUrl(): \Generator {
    yield 'endpoint with a single token replaced' => [
      'https://example.com/api/[form_value:name:field_1]',
      ['field_1' => 'value_1'],
      'https://example.com/api/value_1',
    ];
    yield 'endpoint with multiple tokens replaced' => [
      'https://example.com/api/[form_value:name:field_1]/[form_value:name:field_2]',
      ['field_1' => 'value_1', 'field_2' => 'value_2'],
      'https://example.com/api/value_1/value_2',
    ];
    yield 'returns null when a required token is missing' => [
      'https://example.com/api/[form_value:name:field_1]/[form_value:name:field_2]',
      ['field_1' => '', 'field_2' => 'value_2'],
      NULL,
    ];
  }

  /**
   * Data provider for testGetElementsThatUseOtherFieldsOnApi.
   *
   * @return \Generator
   *   Data to test.
   */
  public static function dataProviderGetElementsThatUseOtherFieldsOnApi(): \Generator {
    yield 'param_name depends on field_2' => [
      [
        'field_1' => [
          '#type' => 'remote_field_markup',
          '#api_endpoint_params' => [
            [
              'param_name' => '[form_value:name:field_2]',
              'param_value' => 'static_value',
            ],
          ],
        ],
      ],
      ['field_2' => ['field_1']],
    ];
    yield 'param_value depends on field_2' => [
      [
        'field_1' => [
          '#type' => 'remote_field_textfield',
          '#api_endpoint_params' => [
            [
              'param_name' => 'param_static',
              'param_value' => '[form_value:name:field_2]',
            ],
          ],
        ],
      ],
      ['field_2' => ['field_1']],
    ];
    yield 'token in api_endpoint' => [
      [
        'field_1' => [
          '#type' => 'remote_field_select',
          '#api_endpoint' => 'https://api.com/[form_value:name:field_2]',
        ],
      ],
      ['field_2' => ['field_1']],
    ];
    yield 'token in api_result_select_value' => [
      [
        'field_1' => [
          '#type' => 'remote_field_checkboxes',
          '#api_result_select_value' => '[form_value:name:field_2]',
        ],
      ],
      ['field_2' => ['field_1']],
    ];
    yield 'token in api_result_select_text' => [
      [
        'field_1' => [
          '#type' => 'remote_field_number',
          '#api_result_select_text' => '[form_value:name:field_2]',
        ],
      ],
      ['field_2' => ['field_1']],
    ];
    yield 'token in headers' => [
      [
        'field_1' => [
          '#type' => 'remote_field_hidden',
          '#headers' => 'Authorization: [form_value:name:field_2]',
        ],
      ],
      ['field_2' => ['field_1']],
    ];
    yield 'container with dependent child' => [
      [
        'group' => [
          '#type' => 'container',
          'field_1' => [
            '#type' => 'remote_field_radios',
            '#api_endpoint' => 'https://api.com/[form_value:name:field_2]',
          ],
        ],
      ],
      ['field_2' => ['field_1']],
    ];
    yield 'non-remote or no type element' => [
      [
        'field_1' => [
          '#type' => 'textfield',
          '#api_endpoint' => '[form_value:name:field_2]',
        ],
        'field_2' => [],
      ],
      [],
    ];
  }

  /**
   * Data provider for extractElementSettings.
   *
   * @return \Generator
   *   Yields test cases for extractElementSettings.
   */
  public static function dataProviderExtractElementSettings(): \Generator {
    yield 'basic settings without headers or tokens' => [
      [
        '#api_endpoint_add_query_parameter' => TRUE,
        '#api_endpoint' => 'https://example.com/api',
        '#api_result_select_value' => 'id',
        '#api_result_select_text' => 'label',
        '#needs_header' => FALSE,
      ],
      [],
      [
        'api_endpoint_add_query_parameter' => TRUE,
        'api_endpoint' => 'https://example.com/api',
        'api_result_select_value' => 'id',
        'api_result_select_text' => 'label',
        'custom_headers' => [],
        'needs_header' => FALSE,
        'api_round_result' => 0,
        'root_property_path' => NULL,
      ],
    ];
    yield 'headers included and parsed' => [
      [
        '#api_endpoint_add_query_parameter' => FALSE,
        '#api_endpoint' => 'https://example.com/api',
        '#api_result_select_value' => 'uuid',
        '#api_result_select_text' => 'name',
        '#needs_header' => TRUE,
        '#headers' => "Authorization: Bearer example_token\nCustom: CustomHeader",
      ],
      [],
      [
        'api_endpoint_add_query_parameter' => FALSE,
        'api_endpoint' => 'https://example.com/api',
        'api_result_select_value' => 'uuid',
        'api_result_select_text' => 'name',
        'custom_headers' => [
          'Authorization' => 'Bearer example_token',
          'Custom' => 'CustomHeader',
        ],
        'needs_header' => TRUE,
        'api_round_result' => 0,
        'root_property_path' => NULL,
      ],
    ];
    yield 'token replaced in select value' => [
      [
        '#api_result_select_value' => '[form_value:name:select_id]',
      ],
      ['select_id' => 'resolved_id'],
      [
        'api_endpoint_add_query_parameter' => FALSE,
        'api_endpoint' => NULL,
        'api_result_select_value' => 'resolved_id',
        'api_result_select_text' => NULL,
        'custom_headers' => [],
        'needs_header' => NULL,
        'api_round_result' => 0,
        'root_property_path' => NULL,
      ],
    ];
  }

}
