<?php

namespace Drupal\Tests\webform_remote_fields\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\webform_remote_fields\Plugin\WebformHandler\WebformRemoteFieldsHandler;
use Drupal\Core\Form\FormState;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Kernel test for WebformRemoteFieldsHandler AJAX logic and form alter.
 *
 * @group webform_remote_fields
 */
class WebformRemoteFieldsHandlerKernelTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'webform',
    'webform_remote_fields',
  ];

  /**
   * Test the AJAX callback for remote fields.
   */
  public function testAlterFormWithRemoteFieldMultipleStep() {
    $form = [
      '#webform_id' => 'remote_field_multiple_step',
      'elements' => [
        // Step 1: choose_product.
        'list_fake_products' => [
          '#type' => 'remote_field_select',
          '#webform_key' => 'list_fake_products',
          '#name' => 'list_fake_products',
          '#access' => TRUE,
          '#webform_parents' => ['choose_product'],
          '#api_endpoint' => 'http://fake-webformremotefields.com/api/products',
          '#api_result_select_value' => 'id',
        ],
        'price' => [
          '#type' => 'remote_field_textfield',
          '#webform_key' => 'price',
          '#name' => 'price',
          '#access' => TRUE,
          '#webform_parents' => ['choose_product', 'product_price'],
          '#api_endpoint' => 'http://fake-webformremotefields.com/api/product/price/[form_value:name:list_fake_products]',
          '#api_result_select_value' => 'price',
        ],
        // Step 2: choose_quantity.
        'quantity' => [
          '#type' => 'number',
          '#webform_key' => 'quantity',
          '#name' => 'quantity',
          '#access' => TRUE,
          '#webform_parents' => ['choose_quantity', 'quantity_of_products'],
        ],
        // Step 3: order_summary.
        'discount' => [
          '#type' => 'remote_field_number',
          '#webform_key' => 'discount',
          '#name' => 'discount',
          '#access' => TRUE,
          '#webform_parents' => ['order_summary'],
          '#api_endpoint' => 'http://fake-webformremotefields.com/api/product/discount/[form_value:name:quantity]',
          '#api_result_select_value' => 'discount',
        ],
      ],
      'progress' => [
        '#current_page' => 'choose_product',
      ],
    ];

    // Mock the helper and cache backend.
    $helper = $this->createMock('Drupal\webform_remote_fields\Service\WebformRemoteFieldHelper');
    // Simulate dependencies: price depends on list_fake_products,
    // discount depends on quantity.
    $helper->method('getElementsThatUseOtherFieldsOnApi')->willReturn([
      'list_fake_products' => [],
      'price' => ['list_fake_products'],
      'quantity' => [],
      'discount' => ['quantity'],
    ]);
    $cache = $this->createMock('Drupal\Core\Cache\CacheBackendInterface');
    $cache->method('get')->willReturn(FALSE);
    $cache->method('set')->willReturn(TRUE);

    $time = $this->createMock('Drupal\Component\Datetime\TimeInterface');
    $time->method('getCurrentTime')->willReturn(time());

    $handler = WebformRemoteFieldsHandler::create(
      $this->container,
      [],
      'webform_remote_fields_handler',
      [],
    );

    $form_state = new FormState();
    $webform_submission = $this->createMock(WebformSubmissionInterface::class);

    $handler->alterForm($form, $form_state, $webform_submission);
    $form_state->setTriggeringElement($form['elements']['list_fake_products']);
    $handler->ajaxCallback($form, $form_state);

    // On the first page,
    // price should have AJAX (depends on list_fake_products, both present).
    $this->assertArrayNotHasKey('#ajax', $form['elements']['price'], 'AJAX callback not added to price.');
    // Field list_fake_products should not have AJAX.
    $this->assertArrayHasKey('#ajax', $form['elements']['list_fake_products'], 'AJAX callback added to list_fake_products.');
    // Field quantity and discount are not on the current page, so no AJAX.
    $this->assertArrayNotHasKey('#ajax', $form['elements']['quantity'], 'AJAX callback not added to quantity.');
    $this->assertArrayNotHasKey('#ajax', $form['elements']['discount'], 'AJAX callback not added to discount.');

    // Move to order_summary page,
    // now discount should have AJAX (depends on quantity, both present).
    $form['progress']['#current_page'] = 'order_summary';
    $form_state->setTriggeringElement($form['elements']['quantity']);
    $form_state->setValue('quantity', 10);
    $handler->alterForm($form, $form_state, $webform_submission);
    $handler->ajaxCallback($form, $form_state);
    $this->assertArrayHasKey('#ajax', $form['elements']['quantity'], 'AJAX callback added to quantity.');
    $this->assertArrayNotHasKey('#ajax', $form['elements']['discount'], 'AJAX callback not added to discount.');
  }

}
