<?php

namespace Drupal\Tests\webform_remote_fields\UnitTest;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Form\FormState;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Tests\UnitTestCase;
use Drupal\webform_remote_fields\Service\HandlerRequest;
use GuzzleHttp\ClientInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Tests the HandlerRequest class.
 *
 * @coversDefaultClass \Drupal\webform_remote_fields\Service\HandlerRequest
 * @group webform_remote_fields
 */
class HandlerRequestTest extends UnitTestCase {

  /**
   * The handler request service.
   */
  protected HandlerRequest $handler;

  /**
   * The request stack service.
   */
  protected RequestStack $requestStack;

  /**
   * The logger channel factory service.
   */
  protected LoggerChannelFactoryInterface $logger;

  /**
   * The cache service.
   */
  protected ConfigFactoryInterface $config;

  /**
   * The Time service.
   */
  protected TimeInterface $time;

  /**
   * The event dispatcher service.
   */
  protected EventDispatcherInterface $eventDispatcher;

  /**
   * The HTTP client service.
   */
  protected ClientInterface $httpClient;

  /**
   * The cache service.
   */
  protected CacheBackendInterface $cache;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $request = $this->createMock(Request::class);
    $request->method('getSchemeAndHttpHost')
      ->willReturn('https://example.com');

    $this->requestStack = $this->createMock(RequestStack::class);
    $this->requestStack->method('getCurrentRequest')
      ->willReturn($request);

    $this->logger = $this->createMock(LoggerChannelFactoryInterface::class);
    $this->logger->method('get')
      ->willReturn($this->createMock(LoggerChannelInterface::class));

    $this->config = $this->createMock(ConfigFactoryInterface::class);
    $this->config->method('get')
      ->willReturn($this->createMock(ImmutableConfig::class));

    $this->time = $this->createMock(TimeInterface::class);

    $this->eventDispatcher = $this->createMock(EventDispatcherInterface::class);
    $this->eventDispatcher->expects($this->any())
      ->method('dispatch')
      ->willReturnCallback(fn($event) => $event);

    $this->httpClient = $this->createMock(ClientInterface::class);
    $this->cache = $this->createMock(CacheBackendInterface::class);

    $this->handler = $this->getMockBuilder(HandlerRequest::class)
      ->setConstructorArgs([
        $this->httpClient,
        $this->logger,
        $this->cache,
        $this->config,
        $this->eventDispatcher,
        $this->requestStack,
        $this->time,
      ])
      ->onlyMethods(['shouldCacheResponse', 'getFromApi', 'cacheApiResponse'])
      ->getMock();
  }

  /**
   * Tests the prepareEndpointUrl method.
   *
   * @covers ::prepareEndpointUrl
   * @dataProvider prepareEndpointUrlProvider
   */
  public function testPrepareEndpointUrl(string $endpoint, string $expected, string $message): void {
    // Create the handler with minimal dependencies.
    $handler = new class(
      $this->httpClient,
      $this->logger,
      $this->cache,
      $this->config,
      $this->eventDispatcher,
      $this->requestStack,
      $this->time
    ) extends HandlerRequest {

      /**
       * Mock method to expose the protected prepareEndpointUrl method.
       *
       * @param string $endpoint
       *   The endpoint to prepare.
       *
       * @return string
       *   The prepared endpoint URL.
       */
      public function callPrepareEndpointUrl(string $endpoint): string {
        return $this->prepareEndpointUrl($endpoint);
      }

    };
    $result = $handler->callPrepareEndpointUrl($endpoint);
    $this->assertEquals($expected, $result, $message);
  }

  /**
   * Data provider for testPrepareEndpointUrl.
   *
   * @return \Generator
   *   Test cases.
   */
  public static function prepareEndpointUrlProvider(): \Generator {
    yield 'internal endpoint' => [
      '[internal]/api/data',
      'https://example.com/api/data',
      'Internal endpoint was not properly converted',
    ];
    yield 'external endpoint' => [
      'https://api.external.com/data',
      'https://api.external.com/data',
      'External endpoint was modified when it should not have been',
    ];
  }

  /**
   * Test the generateCid method.
   *
   * @covers ::generateCid
   * @dataProvider generateCidProvider
   */
  public function testGenerateCid(string $uri, array $options, string $expected, string $message): void {
    $result = HandlerRequest::generateCid($uri, $options);
    $this->assertEquals($expected, $result, $message);
  }

  /**
   * Data provider for testGenerateCid.
   *
   * @return \Generator
   *   Tests cases.
   */
  public static function generateCidProvider(): \Generator {
    yield 'no query' => [
      'https://api.example.com/data',
      [],
      'https://api.example.com/data',
      'Should return URI when no query provided',
    ];
    yield 'empty query' => [
      'https://api.example.com/data',
      ['query' => []],
      'https://api.example.com/data',
      'Should return URI when query is empty',
    ];
    yield 'single query param' => [
      'https://api.example.com/data',
      ['query' => ['foo' => 'bar']],
      'https://api.example.com/data:foo_bar',
      'Should append single query param',
    ];
    yield 'multiple query params' => [
      'https://api.example.com/data',
      ['query' => ['foo' => 'bar', 'baz' => 'qux']],
      'https://api.example.com/data:foo_bar:baz_qux',
      'Should append multiple query params in order',
    ];
    yield 'query param value with space' => [
      'https://api.example.com/data',
      ['query' => ['foo' => 'bar baz']],
      'https://api.example.com/data:foo_bar_baz',
      'Should replace spaces in query param value with underscores',
    ];
    yield 'multiple params with spaces in values' => [
      'https://api.example.com/data',
      ['query' => ['foo' => 'bar baz', 'baz' => 'qux quux']],
      'https://api.example.com/data:foo_bar_baz:baz_qux_quux',
      'Should replace spaces in multiple query param values with underscores',
    ];
    yield 'param name and value with spaces' => [
      'https://api.example.com/data',
      ['query' => ['foo bar' => 'baz qux']],
      'https://api.example.com/data:foo_bar_baz_qux',
      'Should replace spaces in both param name and value with underscores',
    ];
  }

  /**
   * Tests the get method with event subscriber changing params.
   *
   * @covers ::get
   */
  public function testGetEventSubscriberChangesParams(): void {
    // Use a real event and mutate it in the dispatcher mock.
    $eventDispatcher = $this->createMock(EventDispatcherInterface::class);
    $eventDispatcher->expects($this->once())
      ->method('dispatch')
      ->willReturnCallback(function ($event) {
        // Mutate the event as a subscriber would.
        $event->setUri('changed_uri');
        $event->setOptions(['query' => ['foo' => 'bar']]);
        return $event;
      });

    $handler = $this->getMockBuilder(HandlerRequest::class)
      ->setConstructorArgs([
        $this->httpClient,
        $this->logger,
        $this->cache,
        $this->config,
        $eventDispatcher,
        $this->requestStack,
        $this->time,
      ])
      ->onlyMethods(['shouldCacheResponse', 'getFromApi', 'cacheApiResponse'])
      ->getMock();

    $handler->expects($this->once())
      ->method('shouldCacheResponse')
      ->with('changed_uri', ['query' => ['foo' => 'bar']])
      ->willReturn(NULL);

    $handler->expects($this->once())
      ->method('getFromApi')
      ->with('changed_uri', ['query' => ['foo' => 'bar']])
      ->willReturn(['result' => 'ok']);

    $handler->expects($this->once())
      ->method('cacheApiResponse')
      ->with(['result' => 'ok'], 'changed_uri', ['query' => ['foo' => 'bar']]);

    $webformInfo = [
      'form_state' => new FormState(),
      'element' => [],
    ];
    $result = $handler->get('original_uri', ['query' => ['a' => 'b']], $webformInfo);
    $this->assertEquals(['result' => 'ok'], $result, 'Should return API result after event changed params.');
  }

  /**
   * Test get data from a cache.
   *
   * @covers ::get
   */
  public function testGetReturnsCachedData(): void {
    $cached = (object) ['data' => ['cached' => 'value']];
    $this->handler->expects($this->once())
      ->method('shouldCacheResponse')
      ->willReturn($cached);

    $this->handler->expects($this->never())->method('getFromApi');
    $this->handler->expects($this->never())->method('cacheApiResponse');

    $webformInfo = [
      'form_state' => new FormState(),
      'element' => [],
    ];
    $result = $this->handler->get('uri', [], $webformInfo);
    $this->assertEquals(['cached' => 'value'], $result, 'Should return cached data.');
  }

  /**
   * Test getting data from the response and caching it.
   *
   * @covers ::get
   */
  public function testGetCachesApiResponse(): void {
    $this->handler->expects($this->once())
      ->method('shouldCacheResponse')
      ->willReturn(NULL);

    $this->handler->expects($this->once())
      ->method('getFromApi')
      ->willReturn(['api' => 'response']);

    $this->handler->expects($this->once())
      ->method('cacheApiResponse')
      ->with(['api' => 'response'], 'uri', []);

    $webformInfo = [
      'form_state' => new FormState(),
      'element' => [],
    ];
    $result = $this->handler->get('uri', [], $webformInfo);
    $this->assertEquals(['api' => 'response'], $result, 'Should cache and return API response.');
  }

}
